syntax = "proto2";

import "opts.proto";
import "fown.proto";

message reg_file_entry {
	required uint32		id	= 1;
	required uint32		flags	= 2 [(criu).flags = "rfile.flags"];
	required uint64		pos	= 3;
	required fown_entry	fown	= 5;
	required string		name	= 6;
	optional sint32		mnt_id	= 7 [default = -1];
	optional uint64		size	= 8;
	optional bool		ext	= 9;
	optional uint32         mode    = 10;
}
